@@ -65,8 +65,12 @@ lg = LexerGenerator()
 lg.add('SQUOTE_STR', r"(?x)'(?:|[^'\\]|\\.|\\x[0-9a-fA-F]{2}|\\u[0-9a-fA-F]{4}|\\U[0-9a-fA-F]{8})*'")
 lg.add('DQUOTE_STR', r'(?x)"(?:|[^"\\]|\\.|\\x[0-9a-fA-F]{2}|\\u[0-9a-fA-F]{4}|\\U[0-9a-fA-F]{8})*"')
 
+lg.add('SQUOTE_RAW_STR', r"(?x)r'(?:|[^'\\]|\\.|\\x[0-9a-fA-F]{2}|\\u[0-9a-fA-F]{4}|\\U[0-9a-fA-F]{8})*'")
+lg.add('DQUOTE_RAW_STR', r'(?x)r"(?:|[^"\\]|\\.|\\x[0-9a-fA-F]{2}|\\u[0-9a-fA-F]{4}|\\U[0-9a-fA-F]{8})*"')
+
 lg.add('UNTERMINATED_STRING', r"[\"\'].*")
 lg.add('NUMBER', r'-?[0-9]+(?:\.[0-9]+)?')
+lg.add('DOT_NAME', r'\.\&?[_a-zA-Z$][-_a-zA-Z0-9]*')
 lg.add('NAME', r'\&?[_a-zA-Z$][-_a-zA-Z0-9]*')
 lg.add('PIPELINE_FIRST_BIND', r'\|>1\?')
 lg.add('PIPELINE_FIRST', r'\|>1')
@@ -81,7 +85,6 @@ lg.add('LBRACE', r'\{')
 lg.add('RBRACE', r'\}')
 lg.add('LPAREN', r'\(')
 lg.add('RPAREN', r'\)')
-lg.add('DOT', r'\.')
 lg.add('PERCENT', r'%')
 lg.add('COMMA', r',')
 lg.add('THINARROW', r'->')
@@ -155,12 +158,13 @@ klg.add('NOT', r'^not$')
 
 pg = ParserGenerator(['NUMBER', 'OPPLUS', 'OPMINUS', 'OPTIMES', 'OPDIV', 'OPLEQ', 'OPGEQ', 'OPEQ', 'OPNEQ',
                       'OPLT', 'OPGT', 'OPAND', 'OPOR', 'OPIS', 'NOT', 'NEWLINE', 'PERCENT', 'EXPORT',
-                      'LPAREN', 'RPAREN', 'TRUE', 'FALSE', 'DQUOTE_STR', 'SQUOTE_STR', 'AT', 'BANG',
+                      'LPAREN', 'RPAREN', 'TRUE', 'FALSE', 'DQUOTE_STR', 'SQUOTE_STR',
+                      'AT', 'BANG', 'DOT_NAME', 'DQUOTE_RAW_STR', 'SQUOTE_RAW_STR',
                       'NAME', 'EQUALS', 'IF', 'ELSEIF', 'ELSE', 'COLON', 'SEMI', 'DATA', 'IMPORT', 'REQUIRE',
                       'LBRACK', 'RBRACK', 'COMMA', 'DEF', 'DOC', 'CALET', 'PIPELINE', 'PIPELINE_BIND', 'PIPELINE_FIRST',
                       'PIPELINE_FIRST_BIND', 'PIPELINE_SEND', 'PIPELINE_MULTI_SEND', 'RETURN',
                       'LBRACE', 'RBRACE', 'MATCH', 'DEFM', 'RECORD', 'AMP', 'FN', 'THINARROW', 'RECEIVE',
-                      'YIELD', 'FROM', 'FOR', 'IN', 'DOT', 'INDENT', 'DEDENT', 'TRY', 'FINALLY', 'EXCEPT',
+                      'YIELD', 'FROM', 'FOR', 'IN', 'INDENT', 'DEDENT', 'TRY', 'FINALLY', 'EXCEPT',
                       'MODULE', 'AS', 'RAISE', 'WITH', 'MACRO', 'QUOTE', 'QUASI_QUOTE', 'UNQUOTE', 'UNQUOTE_SPLICING'],
                      precedence=[('left', ['EQUALS']),
                                  ('left', ['NOT']),
@@ -244,9 +248,9 @@ def _names_one(p):
     return [p[0].getstr()]
 
 
-@pg.production('_names : _names DOT NAME')
+@pg.production('_names : _names DOT_NAME')
 def _names(p):
-    return p[0] + [p[2].getstr()]
+    return p[0] + [p[1].getstr()[1:]]
 
 
 @pg.production('require_expr : REQUIRE string')
@@ -294,9 +298,9 @@ def tuple_elt(p):
     return p[0]
 
 
-@pg.production('from_expr : FROM names IMPORT NAME')
+@pg.production('from_expr : FROM names IMPORT namelist')
 def from_expr(p):
-    return [Symbol('from-import'), p[1], token_to_symbol(p[3])]
+    return [Symbol('from-import'), p[1], p[3]]
 
 
 @pg.production('suite : binop_expr')  # TODO multi
@@ -479,13 +483,32 @@ def expr_string(p):
 
 
 @pg.production('string : DQUOTE_STR')
-def expr_dquote_str(p):
-    return p[0].getstr()[1:-1]
-
-
 @pg.production('string : SQUOTE_STR')
-def expr_squote_str(p):
-    return p[0].getstr()[1:-1]
+def expr_quote_str(p):
+    string = p[0].getstr()[1:-1]
+    new_string = ''
+    string_enumerator = enumerate(string)
+    for index, char in string_enumerator:
+        if char == '\\':
+            index, char = next(string_enumerator)
+            if char == 'n':
+                char = '\n'
+            elif char == 't':
+                char = '\t'
+            elif char == 'r':
+                char = '\r'
+            elif char in {'\\', "'", '"'}:
+                pass
+            else:
+                char = '\\' + char
+        new_string = new_string + char
+    return new_string
+
+
+@pg.production('string : DQUOTE_RAW_STR')
+@pg.production('string : SQUOTE_RAW_STR')
+def expr_quote_raw_str(p):
+    return p[0].getstr()[2:-1]
 
 
 @pg.production('prim_expr : bool_expr')
@@ -591,9 +614,9 @@ def _compute_underscore_max_num(exps):
     return max_num
 
 
-@pg.production('dot_expr : expr DOT NAME')
+@pg.production('dot_expr : expr DOT_NAME')
 def dot_expr(p):
-    return [Symbol('getattr'), p[0], p[2].getstr()]
+    return [Symbol('getattr'), p[0], p[1].getstr()[1:]]
 
 
 @pg.production('send_msg_expr : expr BANG expr')
@@ -1200,4 +1223,11 @@ def parse(lexer):
             warnings.simplefilter('ignore')
             return pg.build().parse(lexer)
     except ParsingError as e:
-        print("ParsingError: lineno=" + str(e.getsourcepos().lineno) + " colno=" + str(e.getsourcepos().colno))
+        source_pos = e.getsourcepos()
+        if source_pos is None:
+            print('')
+        else:
+            print('ParsingError: lineno='
+                  + str(source_pos.lineno)
+                  + ' colno='
+                  + str(source_pos.colno))
